package com.motrixi.motrixi_flutter

import android.app.Activity
import android.content.Context
import android.util.Log
import androidx.annotation.NonNull
import com.motrixi.datacollection.listener.OnLogListener
import com.motrixi.datacollection.utils.MotrixiSDK
import com.motrixi.datacollection.utils.UploadCollectedData
import io.flutter.embedding.engine.plugins.FlutterPlugin

import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding


/** MotrixiFlutterPlugin */
class MotrixiFlutterPlugin: FlutterPlugin, MethodCallHandler, EventChannel.StreamHandler, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel : MethodChannel
  private var mEventSink: EventChannel.EventSink? = null
  private lateinit var mEventChannel: EventChannel
  private var applicationContext: Context? = null

  private lateinit var activity: Activity

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "motrixi_flutter")
    mEventChannel = EventChannel(flutterPluginBinding.binaryMessenger, "motrixi_flutter_event")
    this.applicationContext = flutterPluginBinding.applicationContext
    channel.setMethodCallHandler(this)
    mEventChannel.setStreamHandler(this)
    addListener()
  }


  protected fun addListener() {
    UploadCollectedData.setOnLogListener(object : OnLogListener {
      override fun onLogListener(s: String) {
        Log.d("---tag--****", s)
        try {
          activity.runOnUiThread {
            mEventSink!!.success(s)
          }
        } catch (e: Exception) {
          Log.e("---error--", e.toString())
        }
      }
    })
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {

    if (call.method == "initMotrixiSDK") {
      val appKey: String = call.argument("appKey") ?: ""
      applicationContext?.let { MotrixiSDK.init(it, appKey) }
      result.success("init Motrixi SDK")
    } else  if (call.method == "uploadSDKData") {
      applicationContext?.let { UploadCollectedData.formatData(it) }
      result.success("Upload Data")
    } else {
      result.notImplemented()
    }

}

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
    mEventChannel.setStreamHandler(null)
  }

  /**
   * Listen
   */
  override fun onListen(arguments: Any?, events: EventChannel.EventSink?) {
    mEventSink = events
  }

  /**
   * Cancel listen
   */
  override fun onCancel(arguments: Any?) {
    mEventSink = null
  }

  override fun onDetachedFromActivity() {
    TODO("Not yet implemented")
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    TODO("Not yet implemented")
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    activity = binding.activity;
  }

  override fun onDetachedFromActivityForConfigChanges() {
    TODO("Not yet implemented")
  }

}


